"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@nodeswork/utils");
const Case = require('case');
class BaseComponentManager {
    constructor(nodeswork) {
        this.classes = [];
        this.nodeswork = nodeswork;
    }
    /**
     * Register a new component class.
     */
    register(cls, options, overwrite = false) {
        let self = this;
        let exists = false;
        cls.prototype.nodeswork = this.nodeswork;
        this.classes.forEach(([clazz, options], index) => {
            if (cls.name == clazz.name) {
                if (overwrite) {
                    self.classes[index] = [clazz, options];
                }
                else {
                    throw new utils_1.NodesworkError(`Component has been registered before,
              set overwrite=true to overwrite it`, { name: cls.name });
                }
                exists = true;
            }
        });
        if (!exists) {
            this.classes.push([cls, options]);
        }
    }
    /**
     * Initialize registered components.
     */
    async initialize() {
        for (let [clazz, options] of this.classes) {
            await clazz.initialize(options);
        }
    }
    /**
     * Call second round of initialized for registered components.
     */
    async initialized() {
        for (let [clazz, options] of this.classes) {
            await clazz.initialized(options);
        }
    }
}
exports.BaseComponentManager = BaseComponentManager;
class NodesworkComponentManager extends BaseComponentManager {
    getComponentMap(ctx) {
        let ret = Object.create(Object.prototype);
        let cache = {};
        let properties = {};
        this.classes.forEach(([clazz, options]) => {
            let name = Case.camel(clazz.name);
            properties[name] = {
                get: () => {
                    if (cache[name] == null) {
                        cache[name] = new clazz(ctx);
                    }
                    return cache[name];
                },
            };
        });
        Object.defineProperties(ret, properties);
        return ret;
    }
}
exports.NodesworkComponentManager = NodesworkComponentManager;

//# sourceMappingURL=component-manager.js.map
