import * as Koa from "koa";
import { Nodeswork } from '../nodeswork';
export declare type NodesworkComponentMap = {
    [name: string]: NodesworkComponent<Koa.Context>;
};
declare module "koa" {
    interface Context {
        components: NodesworkComponentMap;
    }
}
export interface NodesworkComponentOption {
}
export interface NodesworkComponentClass {
    new (ctx: Koa.Context): NodesworkComponent<Koa.Context>;
    initialize(options: NodesworkComponentOption): Promise<void>;
    initialized(options: NodesworkComponentOption): Promise<void>;
}
export declare abstract class NodesworkComponent<CT extends Koa.Context> {
    protected ctx: CT;
    protected nodeswork: Nodeswork;
    constructor(ctx: CT);
    /**
     * Depends on another component.
     *
     * @param {string} name - name of the component which current component
     * depends on.
     * @throws {NodesworkError} when the dependency is missing.
     * @return {NodesworkComponent}
     */
    depends<T extends Koa.Context>(name: string): NodesworkComponent<T>;
    /**
     * Intialize current component when nodeswork starts.
     */
    static initialize(options: NodesworkComponentOption): Promise<void>;
    /**
     * A second round of intialization for dealing with component dependencies.
     */
    static initialized(options: NodesworkComponentOption): Promise<void>;
}
