"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
const logger = require("@nodeswork/logger");
const component_1 = require("./component");
let LOG = logger.getLogger('logger');
class Logger extends component_1.NodesworkComponent {
    getMeta(meta) {
        return _.extend({ key: this.ctx.logKey }, meta);
    }
    request(data) {
        _.extend(this.ctx.requestLog, data);
    }
    debug(message, meta = {}) {
        LOG.debug(message, this.getMeta(meta));
    }
    verbose(message, meta = {}) {
        LOG.verbose(message, this.getMeta(meta));
    }
    info(message, meta = {}) {
        LOG.info(message, this.getMeta(meta));
    }
    warn(message, meta = {}) {
        LOG.warn(message, this.getMeta(meta));
    }
    error(message, meta = {}) {
        LOG.error(message, this.getMeta(meta));
    }
    static async initialize(options) {
        LOG.info('Component Logger finished first round of initialization.');
        this.prototype.nodeswork.use(setupRequestLog);
    }
    static async initialized(options) {
        LOG = logger.getLogger('logger');
        LOG.info('Component Logger is fully initialized.');
    }
}
exports.Logger = Logger;
async function setupRequestLog(ctx, next) {
    ctx.requestLog = {
        header: {
            path: ctx.request.path,
            method: ctx.request.method,
        }
    };
    let responseCode;
    try {
        await next();
        responseCode = ctx.response.status;
    }
    catch (e) {
        responseCode = 500;
        throw e;
    }
    finally {
        ctx.requestLog.header.responseCode = responseCode;
        ctx.components.logger.info('Request', ctx.requestLog);
    }
}

//# sourceMappingURL=logger.js.map
