"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const _ = require("underscore");
const injection_1 = require("./injection");
exports.MIDDLEWARE = 'MIDDLEWARE';
exports.MIDDLEWARE_TARGET_APP = 'app';
exports.MIDDLEWARE_TARGET_ROUTER = 'router';
const MIDDLEWARE_TAGS = ['middleware', 'provider'];
const MIDDLEWARE_METADATA_KEY = Symbol('kiws:middleware');
/**
 * MiddlewareProvider class decorator will register the middlewares declared in
 * the provider to Koa and KoaRouter.
 */
function MiddlewareProvider(options = {}) {
    const injectable = injection_1.Injectable({
        inputs: true,
        tags: _.union(MIDDLEWARE_TAGS, options.tags || []),
        meta: options.meta,
    });
    return (constructor) => {
        injectable(constructor);
        constructor.prototype.$getMiddlewares = $getMiddlewares;
    };
}
exports.MiddlewareProvider = MiddlewareProvider;
function AppMiddleware(options = {}) {
    return middleware(_.extend({}, options, { target: exports.MIDDLEWARE_TARGET_APP }));
}
exports.AppMiddleware = AppMiddleware;
function RouterMiddleware(options = {}) {
    return middleware(_.extend({}, options, { target: exports.MIDDLEWARE_TARGET_ROUTER }));
}
exports.RouterMiddleware = RouterMiddleware;
function middleware(metadata) {
    return (target, propertyName) => {
        metadata.name = propertyName;
        metadata.provider = target.constructor.name;
        const metadatas = Reflect.getOwnMetadata(MIDDLEWARE_METADATA_KEY, target) || [];
        metadatas.push(metadata);
        Reflect.defineMetadata(MIDDLEWARE_METADATA_KEY, metadatas, target);
    };
}
function $getMiddlewares() {
    const metadatas = Reflect.getOwnMetadata(MIDDLEWARE_METADATA_KEY, this.constructor.prototype) || [];
    const middlewares = _.map(metadatas, (metadata) => {
        return _.extend(metadata, { fn: this[metadata.name] });
    });
    return middlewares;
}

//# sourceMappingURL=middleware.js.map
