// Generated by CoffeeScript 2.0.0-beta5
(function() {
  var Mongoose, NAMED, NodesworkComponent, _, logger, registerMongooseModel, validator, winston;

  _ = require('underscore');

  ({logger} = require('@nodeswork/logger'));

  ({validator, NAMED} = require('@nodeswork/utils'));

  winston = require('winston');

  ({NodesworkComponent} = require('../components/component'));

  registerMongooseModel = function(options) {
    var LogSchema, collections, modelName, mongoose;
    ({mongoose, modelName, collections = 'logs'} = options);
    LogSchema = mongoose.Schema({
      timestamp: Date,
      level: String,
      message: String,
      meta: mongoose.Schema.Types.Mixed,
      label: String
    }, {
      collections: collections
    });
    LogSchema.index({
      'meta.key': 1
    });
    LogSchema.statics.searchLog = function(query = {}, options = {}) {
      var limit, res, skip;
      ({limit, skip} = options);
      res = this.find(query).sort({
        timestamp: -1
      });
      if (skip != null) {
        res = res.skip(skip);
      }
      if (limit != null) {
        res = res.limit(limit);
      }
      return res;
    };
    return mongoose.model(modelName, LogSchema);
  };

  // Provide mongoose local database access.
  Mongoose = class Mongoose extends NodesworkComponent {
    constructor(ctx) {
      super(ctx);
    }

    // Intialize Mongoose component. After intialization, Mongoose::mongoose or
    //   instance.mongoose is set to mongoose object.

    // @option options {String} logCollection='logs' specifies the collection under
    //   MongoDB.

    // @throw {NodesworkError} error when nodeswork.config('db') is missing.
    static async initialize(options = {}) {
      var dbURI, logCollection, logQueryPath, storeLog;
      ({storeLog = true, logCollection = 'logs', logQueryPath = '/logs'} = options);
      this.prototype.mongoose = require('mongoose');
      this.prototype.mongoose.Promise = global.Promise;
      dbURI = this.prototype.nodeswork.config('db');
      validator.isRequired(dbURI, {
        meta: {
          path: 'nodeswork.config.db'
        }
      });
      await this.prototype.mongoose.connect(dbURI);
      logger.info('Mongoose is connected.');
      if (storeLog) {
        this.setupLogger(logCollection, logQueryPath);
      }
      logger.info('Mongoose component is initialized.');
    }

    static setupLogger(logCollection, logQueryPath) {
      var Log, MongoDB, db, nwLogger;
      nwLogger = require('@nodeswork/logger');
      ({MongoDB} = require('winston-mongodb'));
      db = this.prototype.mongoose.connections[0].db;
      nwLogger.transports.push(nwLogger.transport(winston.transports.MongoDB, {
        db: db,
        collection: logCollection
      }));
      Log = registerMongooseModel({
        collections: logCollection,
        mongoose: this.prototype.mongoose,
        modelName: 'Log'
      });
      return this.prototype.nodeswork.view(logQueryPath, NAMED('queryLog', _.bind(this.logQueryHandler, this)));
    }

    static async logQueryHandler(ctx, next) {
      var Log, query;
      ({Log} = this.prototype.mongoose.models);
      query = ctx.request.query.key != null ? {
        "meta.key": ctx.request.query.key
      } : {};
      return ctx.body = (await Log.find(query));
    }

  };

  module.exports = {Mongoose};

}).call(this);
