/// <reference types="koa-router" />
/// <reference types="node" />
import * as http from 'http';
import * as Koa from 'koa';
import * as KoaRouter from 'koa-router';
import { RequestAPI, RequestOption } from './globals';
import { NodesworkAccount, AccountKoaContext, NodesworkAccountClass, NodesworkAccountOption, NodesworkAccountManager } from './accounts';
import { Logger, Request, NodesworkComponent, NodesworkComponentClass, NodesworkComponentManager, NodesworkComponentOption } from './components';
export interface NodesworkOption {
    env?: string;
    name?: string;
    server?: string;
    host?: string;
    port?: number;
    appletId?: string;
    appletToken?: string;
    [name: string]: any;
}
export declare type MethodMiddlewaresMap = {
    [path: string]: KoaRouter.IMiddleware[];
};
export declare type NodesworkMiddlewares = {
    [method: string]: MethodMiddlewaresMap;
};
export interface NodesworkContext extends KoaRouter.IRouterContext {
    nodeswork: Nodeswork;
    userApplet: {
        _id: string;
        accounts: {}[];
    };
    execution: any;
    executionId: string;
    logKey: string;
    accounts: NodesworkAccount<AccountKoaContext>[];
}
export interface NodesworkClass {
    new (): Nodeswork;
}
export declare class Nodeswork {
    Nodeswork: typeof Nodeswork;
    Logger: typeof Logger;
    Request: typeof Request;
    NodesworkAccount: typeof NodesworkAccount;
    NodesworkComponent: typeof NodesworkComponent;
    accountManager: NodesworkAccountManager;
    componentManager: NodesworkComponentManager;
    _options: any;
    requestClient: RequestAPI;
    initialized: boolean;
    app: Koa;
    router: KoaRouter;
    server: http.Server;
    middlewares: NodesworkMiddlewares;
    commonMiddlewares: KoaRouter.IMiddleware[];
    constructor(options?: NodesworkOption);
    /**
     * Either to config current nodeswork instance, or retrieve a config value.
     *
     * @param {NodesworkOption} options - config current nodeswork instance.
     * @param {string} options - retrieve the config value for this key.
     */
    config(options?: NodesworkOption | string): Nodeswork | any;
    /**
     * Register a new account.
     */
    withAccount(clazz: NodesworkAccountClass, options: NodesworkAccountOption, overwrite?: boolean): Nodeswork;
    /**
     * Register a new component.
     */
    withComponent(clazz: NodesworkComponentClass, options?: NodesworkComponentOption, overwrite?: boolean): Nodeswork;
    use(...middlewares: KoaRouter.IMiddleware[]): Nodeswork;
    process(...middlewares: KoaRouter.IMiddleware[]): Nodeswork;
    view(viewPath: string, ...middlewares: KoaRouter.IMiddleware[]): Nodeswork;
    action(actionPath: string, ...middlewares: KoaRouter.IMiddleware[]): Nodeswork;
    /**
     * Send request using the client.
     */
    request(options: RequestOption): Promise<any>;
    /**
     * Set the default options for the request client.
     */
    requestDefaults(options: RequestOption): Nodeswork;
    init(): Promise<void>;
    start(): Promise<void>;
    bind(path: string, method: string, ...middlewares: KoaRouter.IMiddleware[]): void;
}
